@@ -143,25 +143,34 @@ class ScopedResolver(FactoryResolver):
         return context[self._resolver_id]
 
 
+FallbackResolver = t.Callable[[t.Any, 'Container'], t.Any]
+
+
 class Container:
-    def __init__(self, parent: Container = None) -> None:
+    def __init__(self, parent: Container = None, fallback_resolver: FallbackResolver = None) -> None:
         self._parent = parent
         self._instances: dict[Key, t.Any] = {}
         self._registry: dict[Key, ServiceResolver] = {}
         self._abs_factories: list[AbstractFactory] = []
         self._abs_factory_cache: dict[Key, AbstractFactory] = {}
+        self._aliases: dict[Key, Key] = {}
+        self._fallback_resolver = fallback_resolver
         self._context: contextvars.ContextVar[t.Optional[dict]] = contextvars.ContextVar('_context', default=None)
 
-    def bind(self, key: Key, instance: t.Any) -> None:
+    def bind(self, key: Key, instance: t.Any, *, alias: Key = None) -> None:
         """Add an instance to the container."""
         self._registry[key] = InstanceResolver(instance)
+        if alias:
+            self.alias(key, alias)
 
-    def factory(
+    def add_factory(
         self,
         key: Key,
         factory: Factory,
+        *,
         scope: Scope = Scope.TRANSIENT,
         initializer: Initializer = None,
+        alias: Key = None,
     ) -> None:
         if scope == Scope.SINGLETON:
             self._registry[key] = SingletonResolver(self, factory, initializer)
@@ -170,17 +179,37 @@ class Container:
         else:
             self._registry[key] = FactoryResolver(self, factory, initializer)
 
-    def add_singleton(self, key: Key, factory: Factory, initializer: Initializer = None) -> None:
-        return self.factory(key, factory, scope=Scope.SINGLETON, initializer=initializer)
+        if alias:
+            self.alias(key, alias)
 
-    def add_scoped(self, key: Key, factory: Factory, initializer: Initializer = None) -> None:
-        return self.factory(key, factory, scope=Scope.SCOPED, initializer=initializer)
+    def add_singleton(self, key: Key, factory: Factory, *, initializer: Initializer = None, alias: Key = None) -> None:
+        return self.factory(key, factory, scope=Scope.SINGLETON, initializer=initializer, alias=alias)
+
+    def add_scoped(self, key: Key, factory: Factory, initializer: Initializer = None, alias: Key = None) -> None:
+        return self.factory(key, factory, scope=Scope.SCOPED, initializer=initializer, alias=alias)
+
+    factory = add_factory
+
+    def alias(self, service: Key, new_name: Key) -> None:
+        self._aliases[new_name] = service
+
+    def fallback(self, resolver: FallbackResolver) -> None:
+        self._fallback_resolver = resolver
+
+    @t.overload
+    def resolve(self, key: t.Type[S]) -> S:
+        ...
+
+    @t.overload
+    def resolve(self, key: t.Any) -> t.Any:
+        ...
 
     def resolve(self, key: t.Any) -> t.Any:
         """Get a service instance from the container.
 
         fixme: https://github.com/python/mypy/issues/4717
         """
+        key = self._get_real_name(key)
         service_resolver = self._registry.get(key)
 
         if service_resolver is None:
@@ -190,11 +219,22 @@ class Container:
 
             abs_factory = self._get_abstract_factory_for(key)
             if abs_factory is None:
-                raise ServiceNotFoundError('Service "%s" is not registered.' % key)
+                if not self._fallback_resolver:
+                    raise ServiceNotFoundError('Service "%s" is not registered.' % key)
+
+                return self._fallback_resolver(key, self)
             return abs_factory.resolve(key, self)
 
         return service_resolver.resolve(self._context.get())
 
+    @t.overload
+    def safe_resolve(self, key: t.Type[S]) -> S:
+        ...
+
+    @t.overload
+    def safe_resolve(self, key: t.Any) -> t.Any:
+        ...
+
     def safe_resolve(self, key: t.Any) -> t.Optional[t.Any]:
         """Return None if service does not exist instead of raising ServiceNotFoundError."""
         try:
@@ -202,6 +242,14 @@ class Container:
         except ServiceNotFoundError:
             return None
 
+    @t.overload
+    def invoke(self, fn_or_class: t.Type[S], extra_kwargs: t.Dict[str, t.Any] = None) -> S:
+        ...
+
+    @t.overload
+    def invoke(self, fn_or_class: t.Callable, extra_kwargs: t.Dict[str, t.Any] = None) -> t.Any:
+        ...
+
     def invoke(self, fn_or_class: t.Union[t.Callable, t.Type], extra_kwargs: t.Dict[str, t.Any] = None) -> t.Any:
         """Invoke a callable resolving and injecting dependencies."""
         injections = {}
@@ -260,6 +308,13 @@ class Container:
                     break
         return self._abs_factory_cache.get(key)
 
+    def _get_real_name(self, key: Key) -> Key:
+        if key in self._aliases:
+            key = self._aliases[key]
+        if self._parent:
+            key = self._parent._get_real_name(key)
+        return key
+
     __setitem__ = bind
     __getitem__ = resolve
 
