@@ -1,6 +1,7 @@
 import typing as t
 
 from kupala.application import get_current_application
+from kupala.contracts import TemplateRenderer
 from kupala.requests import Request
 from kupala.response_factories import ResponseFactory
 
@@ -9,6 +10,6 @@ def response(request: Request, status_code: int = 200, headers: dict = None) ->
     return ResponseFactory(request, status_code, headers)
 
 
-def render_to_string(template_name: str, context: t.Mapping = None) -> str:
+def render_to_string(template_name: str, context: t.Dict = None) -> str:
     app = get_current_application()
-    return app.render(template_name, context)
+    return app.services.resolve(TemplateRenderer).render(template_name, context)
